<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://shapedplugin.com/
 * @since      1.0.0
 *
 * @package    WP_Tabs_Pro
 * @subpackage WP_Tabs_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 */
class WP_Tabs_Pro_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in WP_Tabs_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The WP_Tabs_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-tabs-pro-admin.min.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the widget for the public-facing side of the site.
	 *
	 * @param arr $widget The widget of the plugin.
	 * @since    2.0.1
	 */
	public function register_wptabs_widget( $widget ) {
		register_widget( 'WP_Tabs_Pro_Widget' );
		return $widget;
	}

	/**
	 * Function creates tabs duplicate as a draft.
	 */
	public function duplicate_wp_tabs() {
		global $wpdb;
		if ( ! ( isset( $_GET['post'] ) || isset( $_POST['post'] ) || ( isset( $_REQUEST['action'] ) && 'sp_duplicate_tabs' == $_REQUEST['action'] ) ) ) { // phpcs:ignore
			wp_die( esc_html__( 'No tabs to duplicate has been supplied!', 'wp-tabs-pro' ) );
		}

		/**
		* Nonce verification
		*/
		if ( ! isset( $_GET['sp_duplicate_tabs_nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['sp_duplicate_tabs_nonce'] ) ), basename( __FILE__ ) ) ) {
			return;
		}

		/**
		* Get the original shortcode id
		*/
		$post_id    = ( isset( $_GET['post'] ) ? absint( $_GET['post'] ) : absint( $_POST['post'] ) );
		$capability = apply_filters( 'sp_wp_tab_pro_ui_permission', 'manage_options' );
		$show_ui    = current_user_can( $capability ) ? true : false;
		if ( ! $show_ui && get_post_type( $post_id ) !== 'sp_wp_tabs' ) {
			wp_die( esc_html__( 'No shortcode to duplicate has been supplied!', 'wp-tabs-pro' ) );
		}
		/**
		* And all the original shortcode data then
		*/
		$post = get_post( $post_id );

		$current_user    = wp_get_current_user();
		$new_post_author = $current_user->ID;

		/**
		* If shortcode data exists, create the shortcode duplicate
		*/
		if ( isset( $post ) && null !== $post ) {

			/**
			* New shortcode data array
			*/
			$args = array(
				'comment_status' => $post->comment_status,
				'ping_status'    => $post->ping_status,
				'post_author'    => $new_post_author,
				'post_content'   => $post->post_content,
				'post_excerpt'   => $post->post_excerpt,
				'post_name'      => $post->post_name,
				'post_parent'    => $post->post_parent,
				'post_password'  => $post->post_password,
				'post_status'    => 'draft',
				'post_title'     => $post->post_title,
				'post_type'      => $post->post_type,
				'to_ping'        => $post->to_ping,
				'menu_order'     => $post->menu_order,
			);

			/**
			* Insert the shortcode by wp_insert_post() function
			*/
			$new_post_id = wp_insert_post( $args );

			/**
			* Get all current post terms ad set them to the new post draft
			*/
			$taxonomies = get_object_taxonomies( $post->post_type ); // Returns array of taxonomy names for post type, ex array("category", "post_tag");.
			foreach ( $taxonomies as $taxonomy ) {
				$post_terms = wp_get_object_terms( $post_id, $taxonomy, array( 'fields' => 'slugs' ) );
				wp_set_object_terms( $new_post_id, $post_terms, $taxonomy, false );
			}

			$post_meta_infos = get_post_custom( $post_id );
			// Duplicate all post meta.
			foreach ( $post_meta_infos as $key => $values ) {
				foreach ( $values as $value ) {
					$value = wp_slash( maybe_unserialize( $value ) ); // Unserialize data to avoid conflicts.
					add_post_meta( $new_post_id, $key, $value );
				}
			}
			// Finally, redirect to the edit post screen for the new draft.
			wp_safe_redirect( esc_url( admin_url( 'edit.php?post_type=' . $post->post_type ) ) );

			exit;
		} else {
			wp_die( esc_html__( 'Tabs creation failed, could not find original tabs: ', 'wp-tabs-pro' ) . esc_attr( $post_id ) );
		}
	}

	/**
	 * Add the duplicate link to action list for post_row_actions
	 */
	/**
	 * Add the duplicate link to action list for post_row_actions
	 *
	 * @param  mixed $actions action.
	 * @param  mixed $post post type.
	 * @return statement
	 */
	public function sp_duplicate_tabs_link( $actions, $post ) {
		$capability = apply_filters( 'sp_wp_tab_pro_ui_permission', 'manage_options' );
		$show_ui    = current_user_can( $capability ) ? true : false;
		if ( $show_ui && 'sp_wp_tabs' === $post->post_type ) {
			$actions['duplicate'] = '<a href="' . wp_nonce_url( 'admin.php?action=sp_duplicate_tabs&post=' . $post->ID, basename( __FILE__ ), 'sp_duplicate_tabs_nonce' ) . '" rel="permalink">' . __( 'Duplicate', 'wp-tabs-pro' ) . '</a>';
		}
		return $actions;
	}

	/**
	 * Declare the compatibility of WooCommerce High-Performance Order Storage (HPOS) feature.
	 *
	 * @since 2.2.0
	 *
	 * @return void
	 */
	public function declare_compatibility_with_woo_hpos_feature() {
		if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', 'wp-tabs-pro/wp-tabs-pro.php', true );
		}
	}

}
