<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://shapedplugin.com/
 * @since      1.0.0
 *
 * @package    WP_Tabs_Pro
 * @subpackage WP_Tabs_Pro/includes
 * @author     ShapedPlugin <help@shapedplugin.com>
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 */
class SP_WP_Tabs_Pro {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      WP_Tabs_Pro_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		// The code that runs during plugin updates.
		require_once WP_TABS_PRO_PATH . 'includes/class-wp-tabs-pro-updates.php';

		if ( defined( 'WP_TABS_PRO_VERSION' ) ) {
			$this->version = WP_TABS_PRO_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wp-tabs-pro';

		/**
		 * ElementOr shortcode block.
		 */
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
		if ( ( is_plugin_active( 'elementor/elementor.php' ) || is_plugin_active_for_network( 'elementor/elementor.php' ) ) ) {
			require_once WP_TABS_PRO_PATH . 'admin/ElementorAddons/class-wp-tabs-pro-elementor-addons.php';
			new WP_Tabs_Pro_Elementor_Addons();
		}

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		$this->sptpro_wc_tab();

	}

	/**
	 * Register WooCommerce hooks.
	 *
	 * @since 2.0.2
	 * @access private
	 */
	private function sptpro_wc_tab() {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
		$settings       = get_option( 'sp-tab__settings' );
		$sptpro_woo_tab = isset( $settings['sptpro_woo_tab'] ) ? $settings['sptpro_woo_tab'] : false;

		if ( ( $sptpro_woo_tab ) && ( is_plugin_active( 'woocommerce/woocommerce.php' ) || is_plugin_active_for_network( 'woocommerce/woocommerce.php' ) ) ) {
			$sptpro_woo_set_tabs = isset( $settings['sptpro_woo_set_tab'] ) ? $settings['sptpro_woo_set_tab'] : array();
			if ( $sptpro_woo_set_tabs ) {
				foreach ( $sptpro_woo_set_tabs as $sptpro_woo_set_tab ) {
					// Product tab.
					$product_tab = new WP_Tabs_Pro_Product_Tab( $sptpro_woo_set_tab );

					$this->loader->add_filter( 'woocommerce_product_tabs', $product_tab, 'sptpro_woo_tab', 10, 2 );
				}
			}
		}
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - WP_Tabs_Pro_Loader. Orchestrates the hooks of the plugin.
	 * - WP_Tabs_Pro_i18n. Defines internationalization functionality.
	 * - WP_Tabs_Pro_Admin. Defines all hooks for the admin area.
	 * - WP_Tabs_Pro_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-tabs-pro-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-tabs-pro-i18n.php';

		/**
		 * The class responsible for custom post type
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-tabs-pro-cpt.php';

		/**
		 * The class responsible for admin sub-menu
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-tabs-pro-admin-menu.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wp-tabs-pro-admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/class-wp-tabs-pro-widget.php';

		/**
		 * The class responsible for shortcode
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wp-tabs-pro-shortcode.php';

		/**
		 * The class live preview.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/preview/class-wp-tabs-pro-preview.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wp-tabs-pro-public.php';

		$this->loader = new WP_Tabs_Pro_Loader();
		require_once WP_TABS_PRO_PATH . 'includes/class-wp-tabs-pro-import-export.php';
		require_once WP_TABS_PRO_PATH . 'admin/helper/class-wp-tabs-pro-cron.php';
		require_once WP_TABS_PRO_PATH . 'includes/class-wp-tabs-pro-license.php';

		require_once WP_TABS_PRO_PATH . 'includes/class-wp-tabs-pro-product-tab.php';

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the WP_Tabs_Pro_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new WP_Tabs_Pro_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		// Plugin admin styles n scripts.
		$plugin_admin = new WP_Tabs_Pro_Admin( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'widgets_init', $plugin_admin, 'register_wptabs_widget' );
		$this->loader->add_action( 'admin_action_sp_duplicate_tabs', $plugin_admin, 'duplicate_wp_tabs' );
		$this->loader->add_filter( 'post_row_actions', $plugin_admin, 'sp_duplicate_tabs_link', 10, 2 );
		$this->loader->add_action( 'before_woocommerce_init', $plugin_admin, 'declare_compatibility_with_woo_hpos_feature' );

		// Plugin admin custom post types.
		$plugin_admin_cpt = new WP_Tabs_Pro_CPT( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'init', $plugin_admin_cpt, 'sptpro_post_type' );
		$this->loader->add_filter( 'post_updated_messages', $plugin_admin_cpt, 'sptpro_updated_messages', 10, 2 );
		$this->loader->add_filter( 'manage_sp_wp_tabs_posts_columns', $plugin_admin_cpt, 'sptpro_admin_column' );
		$this->loader->add_action( 'manage_sp_wp_tabs_posts_custom_column', $plugin_admin_cpt, 'sptpro_admin_field', 10, 2 );

		// Plugin admin menus.
		$plugin_admin_menu = new WP_Tabs_Pro_Admin_Menu( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'admin_menu', $plugin_admin_menu, 'sptpro_help_admin_submenu', 100 );
		$this->loader->add_filter( 'plugin_action_links', $plugin_admin_menu, 'sptpro_plugin_action_links', 10, 2 );
		$this->loader->add_filter( 'admin_footer_text', $plugin_admin_menu, 'sptpro_review_text', 10, 2 );

		// Export Import.
		$import_export = new Wp_Tabs_Pro_Import_Export( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'wp_ajax_tabs_export_shortcode', $import_export, 'export_shortcode' );
		$this->loader->add_action( 'wp_ajax_tabs_import_shortcode', $import_export, 'import_shortcode' );

		// License Page.
		$manage_license = new WP_Tabs_Pro_License( WP_TABS_PRO_FILE, WP_TABS_PRO_VERSION, 'ShapedPlugin', WP_TABS_PRO_STORE_URL, WP_TABS_PRO_ITEM_ID, WP_TABS_PRO_ITEM_SLUG );

		// Admin Menu.
		$this->loader->add_action( 'admin_init', $manage_license, 'wp_tabs_pro_activate_license' );
		$this->loader->add_action( 'admin_init', $manage_license, 'wp_tabs_pro_deactivate_license' );

		$this->loader->add_action( 'wp_tabs_pro_weekly_scheduled_events', $manage_license, 'check_license_status' );

		// Init Updater.
		$this->loader->add_action( 'admin_init', $manage_license, 'init_updater', 0 );

		// Display notices to admins.
		$this->loader->add_action( 'admin_notices', $manage_license, 'license_active_notices' );
		$this->loader->add_action( 'in_plugin_update_message-' . WP_TABS_PRO_BASENAME, $manage_license, 'plugin_row_license_missing', 10, 2 );

		// Redirect after active.
		$this->loader->add_action( 'activated_plugin', $this, 'redirect_to' );

		/**
		 * Gutenberg block.
		 */
		if ( version_compare( $GLOBALS['wp_version'], '5.3', '>=' ) ) {
			require_once WP_TABS_PRO_PATH . 'admin/class-wp-tabs-pro-gutenberg-block.php';
			new WP_Tabs_Pro_Gutenberg_Block();
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		// Plugin public enqueue.
		$plugin_public = new WP_Tabs_Pro_Public( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'wp_loaded', $plugin_public, 'register_scripts' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'front_enqueue_style' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_public, 'admin_enqueue_scripts' );
		$this->loader->add_action( 'save_post', $plugin_public, 'delete_page_sp_tabs_option_on_save' );

		// Add Shortcode.
		$plugin_shortcode = new WP_Tabs_Pro_Shortcode( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'sptpro_action_tag_for_shortcode', $plugin_shortcode, 'sptpro_shortcode_execute' );
		add_shortcode( 'wptabs', array( $plugin_shortcode, 'sptpro_shortcode_execute' ) );

	}

	/**
	 * Redirect after active.
	 *
	 * @param string $plugin The plugin slug.
	 * @return void
	 */
	public function redirect_to( $plugin ) {
		$manage_license     = new WP_Tabs_Pro_License( WP_TABS_PRO_FILE, WP_TABS_PRO_VERSION, 'ShapedPlugin', WP_TABS_PRO_STORE_URL, WP_TABS_PRO_ITEM_ID, WP_TABS_PRO_ITEM_SLUG );
		$license_key_status = $manage_license->get_license_status();
		$license_status     = ( is_object( $license_key_status ) ? $license_key_status->license : '' );
		if ( WP_TABS_PRO_BASENAME === $plugin && 'valid' !== $license_status ) {
			exit( wp_redirect( admin_url( 'edit.php?post_type=sp_wp_tabs&page=tabs_settings#tab=1' ) ) ); // phpcs:ignore
		}
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    WP_Tabs_Pro_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
