<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://shapedplugin.com/
 * @since      1.0.0
 *
 * @package    WP_Tabs_Pro
 * @subpackage WP_Tabs_Pro/public
 * @author     ShapedPlugin <help@shapedplugin.com>
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 */
class WP_Tabs_Pro_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $minify scripts.
	 */
	private $min;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->min         = ( apply_filters( 'enqueue_dev_mode', false ) || WP_DEBUG ) ? '' : '.min';
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function register_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in WP_Tabs_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The WP_Tabs_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$settings              = get_option( 'sp-tab__settings' );
		$enqueue_font_awesome  = isset( $settings['sptpro_dequeue_fa_css'] ) ? $settings['sptpro_dequeue_fa_css'] : true;
		$enqueue_animation_css = isset( $settings['sptpro_dequeue_animation_css'] ) ? $settings['sptpro_dequeue_animation_css'] : true;
		if ( $enqueue_font_awesome ) {
			wp_register_style( 'sptpro-font-awesome', WP_TABS_PRO_URL . 'public/css/font-awesome.min.css', array(), $this->version );
		}
		wp_register_style( 'sptpro-accordion-style', WP_TABS_PRO_URL . 'public/css/sptpro-accordion' . $this->min . '.css', array(), $this->version, 'all' );
		wp_register_style( 'sptpro-tab-carousel-style', WP_TABS_PRO_URL . 'public/css/slick-theme.min.css', array(), $this->version, 'all' );
		wp_register_style( 'sptpro-tab-carousel-theme-style', WP_TABS_PRO_URL . 'public/css/slick.min.css', array(), $this->version, 'all' );
		if ( $enqueue_animation_css ) {
			wp_register_style( 'sptpro-animate-css', WP_TABS_PRO_URL . 'public/css/animate' . $this->min . '.css', array(), $this->version, 'all' );
		}
		wp_register_style( 'sptpro-style', WP_TABS_PRO_URL . 'public/css/wp-tabs-pro-public' . $this->min . '.css', array(), $this->version, 'all' );

		// Register scripts.
		wp_register_script( 'sptpro-tab', WP_TABS_PRO_URL . 'public/js/tab' . $this->min . '.js', array( 'jquery' ), $this->version, false );
		wp_register_script( 'sptpro-collapse', WP_TABS_PRO_URL . 'public/js/collapse' . $this->min . '.js', array( 'jquery' ), $this->version, false );
		wp_register_script( 'sptpro-smooth-scrollbar', WP_TABS_PRO_URL . 'public/js/smooth-scrollbar.js', null, $this->version, null );
		wp_register_script( 'sptpro-match-height', WP_TABS_PRO_URL . 'public/js/jquery.matchHeight-min.js', array( 'jquery' ), $this->version, false );
		wp_register_script( 'sptpro-tab-carousel-script', WP_TABS_PRO_URL . 'public/js/slick.min.js', array( 'jquery' ), $this->version, true );
		wp_register_script( 'sptpro-script', WP_TABS_PRO_URL . 'public/js/wp-tabs-pro-public' . $this->min . '.js', array( 'jquery' ), $this->version, true );
		$custom_js = isset( $settings['sptpro_custom_js'] ) ? trim( html_entity_decode( $settings['sptpro_custom_js'] ) ) : '';
		if ( ! empty( $custom_js ) ) {
			wp_add_inline_script( 'sptpro-script', $custom_js );
		}
	}

	/**
	 * Enqueue the stylesheets and scripts for the live preview.
	 *
	 * @since    2.0.10
	 */
	public function front_enqueue_style() {
		// Stylesheet loading problem solving here. Shortcode id to push page id option for getting how many shortcode in the page.
		$current_page_id    = get_queried_object_id();
		$option_key         = 'sp_tabs_page_id' . $current_page_id;
		$found_generator_id = get_option( $option_key );
		if ( is_multisite() ) {
			$option_key         = 'sp_tabs_page_id' . get_current_blog_id() . $current_page_id;
			$found_generator_id = get_site_option( $option_key );
		}
		if ( $found_generator_id ) {
			wp_enqueue_style( 'sptpro-font-awesome' );
			wp_enqueue_style( 'sptpro-animate-css' );
			wp_enqueue_style( 'sptpro-tab-carousel-style' );
			wp_enqueue_style( 'sptpro-tab-carousel-theme-style' );
			wp_enqueue_style( 'sptpro-style' );

			$sptpro_dynamic_style = '';
			$sptpro_typography    = array();
			foreach ( $found_generator_id as $post_id ) {
				if ( $post_id && is_numeric( $post_id ) && get_post_status( $post_id ) !== 'trash' ) {
					$sptpro_data_upload       = get_post_meta( $post_id, 'sp_tab_source_options', true );
					$sptpro_shortcode_options = get_post_meta( $post_id, 'sp_tab_shortcode_options', true );
					include WP_TABS_PRO_PATH . 'public/dynamic_style.php';
				}
			}
			/**
			* Google font link enqueue
			*/
			if ( $sptpro_dequeue_google_font ) {
				$enqueue_fonts = self::load_google_fonts( $sptpro_typography );
				if ( ! empty( $enqueue_fonts ) ) {
					wp_enqueue_style( 'sptpro-google-fonts', esc_url( add_query_arg( 'family', rawurlencode( implode( '|', $enqueue_fonts ) ), '//fonts.googleapis.com/css' ) ), array(), '2.0.0' );
				}
			}

			$settings              = get_option( 'sp-tab__settings' );
			$custom_style          = isset( $settings['sptpro_custom_css'] ) ? trim( html_entity_decode( $settings['sptpro_custom_css'] ) ) : '';
			$sptpro_dynamic_style .= $custom_style;
			$dynamic_style         = self::minify_output( $sptpro_dynamic_style );
			wp_add_inline_style( 'sptpro-style', $dynamic_style );
		}
	}

	/**
	 * Delete page shortcode ids array option on save
	 *
	 * @param  int $post_ID current post id.
	 * @return void
	 */
	public function delete_page_sp_tabs_option_on_save( $post_ID ) {
		if ( is_multisite() ) {
			$option_key = 'sp_tabs_page_id' . get_current_blog_id() . $post_ID;
			if ( get_site_option( $option_key ) ) {
				delete_site_option( $option_key );
			}
		} elseif ( get_option( 'sp_tabs_page_id' . $post_ID ) ) {
				delete_option( 'sp_tabs_page_id' . $post_ID );
		}
	}

	/**
	 * Enqueue the stylesheets and scripts for the live preview.
	 *
	 * @since    2.0.10
	 */
	public function admin_enqueue_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in WP_Tabs_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The WP_Tabs_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$current_screen        = get_current_screen();
		$the_current_post_type = $current_screen->post_type;
		if ( 'sp_wp_tabs' === $the_current_post_type ) {
			wp_enqueue_style( 'sptpro-font-awesome' );
			wp_enqueue_style( 'sptpro-accordion-style' );
			wp_enqueue_style( 'sptpro-animate-css' );
			wp_enqueue_style( 'sptpro-tab-carousel-style' );
			wp_enqueue_style( 'sptpro-tab-carousel-theme-style' );
			wp_enqueue_style( 'sptpro-style' );

			// Public Scripts for backend Preview.
			wp_enqueue_script( 'sptpro-tab' );
			wp_enqueue_script( 'sptpro-collapse' );
			wp_enqueue_script( 'sptpro-smooth-scrollbar' );
			wp_enqueue_script( 'sptpro-match-height' );
			wp_enqueue_script( 'sptpro-tab-carousel-script' );
		}
	}

	/**
	 * The load google fonts function marge all fonts from shortcodes.
	 *
	 * @param  array $sptpro_typography store the all shortcode typography.
	 * @return array
	 */
	public static function load_google_fonts( $sptpro_typography ) {
		$enqueue_fonts = array();
		if ( ! empty( $sptpro_typography ) ) {
			foreach ( $sptpro_typography as $font ) {
				if ( isset( $font['type'] ) && 'google' === $font['type'] ) {
					$variant         = isset( $font['font-weight'] ) ? ':' . $font['font-weight'] : '';
					$subset          = isset( $font['subset'] ) ? ':' . $font['subset'] : '';
					$enqueue_fonts[] = $font['font-family'] . $variant . $subset;
				}
			}
		}
		return $enqueue_fonts;
	}

	/**
	 * Minify output
	 *
	 * @param  string $html output.
	 * @return statement
	 */
	public static function minify_output( $html ) {
		$html = preg_replace( '/<!--(?!s*(?:[if [^]]+]|!|>))(?:(?!-->).)*-->/s', '', $html );
		$html = str_replace( array( "\r\n", "\r", "\n", "\t" ), '', $html );
		while ( stristr( $html, '  ' ) ) {
			$html = str_replace( '  ', ' ', $html );
		}
		return $html;
	}

}
